"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteCollection = exports.deleteDocument = exports.getData = exports.addData = exports.initialize = void 0;
const app_1 = require("firebase-admin/app");
const firestore_1 = require("firebase-admin/firestore");
let app;
let db;
const initialize = (config) => __awaiter(void 0, void 0, void 0, function* () {
    if (!app || !db) {
        app = yield (0, app_1.initializeApp)({
            credential: (0, app_1.cert)(config),
        });
        db = yield (0, firestore_1.getFirestore)(app);
    }
});
exports.initialize = initialize;
function addData(collectionName, documentName, dataObj) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!app || !db) {
            throw new Error("Firebase is not initialized. Please initialize firebase before trying to access any of its service.");
        }
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const docName = typeof documentName === "function" ? documentName() : documentName;
        try {
            const dataRef = yield db.collection(colName).doc(docName);
            yield dataRef.set(dataObj);
            return true;
        }
        catch (error) {
            console.error(error);
            return false;
        }
    });
}
exports.addData = addData;
function getData(collectionName, documentName) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!app || !db) {
            throw new Error("Firebase is not initialized. Please initialize firebase before trying to access any of its service.");
        }
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const docName = typeof documentName === "function" ? documentName() : documentName;
        try {
            const dataRef = db.collection(colName).doc(docName);
            const doc = yield dataRef.get();
            if (!doc.exists) {
                return false;
            }
            else {
                return doc.data();
            }
        }
        catch (error) {
            console.error(error);
            return false;
        }
    });
}
exports.getData = getData;
function deleteDocument(collectionName, documentName) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!app || !db) {
            throw new Error("Firebase is not initialized. Please initialize firebase before trying to access any of its service.");
        }
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const docName = typeof documentName === "function" ? documentName() : documentName;
        try {
            yield db.collection(colName).doc(docName).delete();
            return true;
        }
        catch (error) {
            console.error(error);
            return false;
        }
    });
}
exports.deleteDocument = deleteDocument;
const deleteBatches = (query) => __awaiter(void 0, void 0, void 0, function* () {
    const snapshot = yield query.get();
    const batchSize = snapshot.size;
    if (batchSize === 0) {
        return true;
    }
    const batch = db.batch();
    snapshot.docs.forEach((doc) => {
        batch.delete(doc.ref);
    });
    yield batch.commit();
    process.nextTick(() => {
        deleteBatches(query);
    });
});
function deleteCollection(collectionName, batchSize) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!app || !db) {
            throw new Error("Firebase is not initialized. Please initialize firebase before trying to access any of its service.");
        }
        const colName = typeof collectionName === "function" ? collectionName() : collectionName;
        const batch = typeof batchSize === "function" ? batchSize() : batchSize;
        try {
            const collectionRef = yield db.collection(colName);
            const query = yield collectionRef.orderBy("__name__").limit(batch);
            yield deleteBatches(query);
            return true;
        }
        catch (error) {
            console.error(error);
            return false;
        }
    });
}
exports.deleteCollection = deleteCollection;
